package model

import (
	"time"

	"gitea.dwysokinski.me/twhelp/dcbot/internal/domain"
	"github.com/google/uuid"
	"github.com/uptrace/bun"
)

type Group struct {
	bun.BaseModel `bun:"base_model,table:groups,alias:group"`

	ID                    uuid.UUID `bun:"id,type:uuid,pk,default:gen_random_uuid()"`
	ServerID              string    `bun:"server_id,type:varchar(100),notnull"`
	ChannelGainedVillages string    `bun:"channel_gained_villages,type:varchar(100),nullzero"`
	ChannelLostVillages   string    `bun:"channel_lost_villages,type:varchar(100),nullzero"`
	ServerKey             string    `bun:"server_key,type:varchar(50),notnull"`
	VersionCode           string    `bun:"version_code,type:varchar(6),notnull"`
	CreatedAt             time.Time `bun:"created_at,nullzero,notnull,default:current_timestamp"`
}

func (g Group) ToDomain() domain.Group {
	return domain.Group{
		ID:                    g.ID.String(),
		ServerID:              g.ServerID,
		ChannelGainedVillages: g.ChannelGainedVillages,
		ChannelLostVillages:   g.ChannelLostVillages,
		ServerKey:             g.ServerKey,
		VersionCode:           g.VersionCode,
		CreatedAt:             g.CreatedAt,
	}
}
