package twhelp

type ErrorCode string

const (
	ErrorCodeInternalServerError ErrorCode = "internal-server-error"
	ErrorCodeEntityNotFound      ErrorCode = "entity-not-found"
	ErrorCodeValidationError     ErrorCode = "validation-error"
	ErrorCodeRouteNotFound       ErrorCode = "route-not-found"
	ErrorCodeMethodNotAllowed    ErrorCode = "method-not-allowed"
)

func (c ErrorCode) String() string {
	return string(c)
}

type APIError struct {
	Code    ErrorCode `json:"code"`
	Message string    `json:"message"`
}

func (e APIError) Error() string {
	return e.Message + " (code=" + e.Code.String() + ")"
}

type errorResp struct {
	Error APIError `json:"error"`
}

type Version struct {
	Code     string `json:"code"`
	Host     string `json:"host"`
	Name     string `json:"name"`
	Timezone string `json:"timezone"`
}

type listVersionsResp struct {
	Data []Version `json:"data"`
}

type Server struct {
	Key  string `json:"key"`
	URL  string `json:"url"`
	Open bool   `json:"open"`
}

type getServerResp struct {
	Data Server `json:"data"`
}
